
#include <Wire.h>
#define __PROG_TYPES_COMPAT__
#include <avr/pgmspace.h>
#include "maze.h"
#include "Adafruit_SI1145.h"
#include "SparkFun_VL6180X.h"


#define AIN1 A3 //PF4
#define AIN2 A2 //PF5
#define BIN1 A1 //PF6
#define BIN2 A0 //PF7
#define PWMA 6  //PD0
#define PWMB 5  //PC6
#define STBY 8  //PD1

uint16_t minSpeed = 70; //minimum speed to get motors turning at an average pace
uint16_t tSpeed = 100; //turn speed
static float currentAngle = 0; //initialize the gyro
static long then = millis(); //start recording the time that has passed for the FIRST integration of the gyro
int16_t zRate; //initialize z axis of the gyro

//initialize the UV sensors
Adafruit_SI1145 uv = Adafruit_SI1145();
Adafruit_SI1145 uv1 = Adafruit_SI1145();

const uint16_t black = 470; //constant for determining black surfaces

//This is a list of registers in the ITG-3200. Registers are parameters that determine how the sensor will behave, or they can hold data that represent the
//sensors current status.
//To learn more about the registers on the ITG-3200, download and read the datasheet.
char WHO_AM_I = 0x00;
char SMPLRT_DIV = 0x15;
char DLPF_FS = 0x16;
char GYRO_XOUT_H = 0x1D;
char GYRO_XOUT_L = 0x1E;
char GYRO_YOUT_H = 0x1F;
char GYRO_YOUT_L = 0x20;
char GYRO_ZOUT_H = 0x21;
char GYRO_ZOUT_L = 0x22;

//This is a list of settings that can be loaded into the registers.
//DLPF, Full Scale Register Bits
//FS_SEL must be set to 3 for proper operation
//Set DLPF_CFG to 3 for 1kHz Fint and 42 Hz Low Pass Filter
char DLPF_CFG_0 = (1 << 0);
char DLPF_CFG_1 = (1 << 1);
char DLPF_CFG_2 = (1 << 2);
char DLPF_FS_SEL_0 = (1 << 3);
char DLPF_FS_SEL_1 = (1 << 4);

//I2C devices each have an address. The address is defined in the datasheet for the device. The ITG-3200 breakout board can have different address depending on how
//the jumper on top of the board is configured. By default, the jumper is connected to the VDD pin. When the jumper is connected to the VDD pin the I2C address
//is 0x69.
char itgAddress = 0x69;
#define MODE 0x40 //address for controlling Bluetooth modes
#define TCAADDR 0x70 //address for the I2C multiplexer
#define VL6180X_ADDRESS 0x29 //range finder address

VL6180x sensor(VL6180X_ADDRESS);

//multiplexer selection function
void tcaselect(uint8_t i) {
  if (i > 7) return;

  Wire.beginTransmission(TCAADDR);
  Wire.write(1 << i);
  Wire.endTransmission();
}

void setup() {
  //LEDs
  pinMode(10, OUTPUT);
  pinMode(14, OUTPUT);
  
  Serial.begin(9600);
  Init2Winit();
  Serial.println("Adafruit SI1145 test");
  Serial.println("OK!");

  //Read the WHO_AM_I register and print the result
  char id = 0;
  id = itgRead(itgAddress, 0x00);
  Serial.print("ID: ");
  Serial.println(id, HEX);

  //Configure the gyroscope
  //Set the gyroscope scale for the outputs to +/-2000 degrees per second
  itgWrite(itgAddress, DLPF_FS, (DLPF_FS_SEL_0 | DLPF_FS_SEL_1 | DLPF_CFG_0));
  //Set the sample rate to 100 hz
  itgWrite(itgAddress, SMPLRT_DIV, 9);

  //initialize sensor 1
  tcaselect(0);
  uv.begin();

  //initialize sensor 2
  tcaselect(1);
  uv1.begin();

  if (sensor.VL6180xInit() != 0) {
    Serial.println("FAILED TO INITALIZE"); //Initialize device and check for errors
  }
  delay(5000); //wait 5 seconds when switched on to position the robot
}

void loop() {
  //structure initialization
  static MyRobot_t test{0x03 , 0x00, 0x12, 0x00, 0x00, 0x00};
  
  
  static bool isIntersect; //state variables determining if at intersection for WhichWay
  static bool lastState;

  sensor.VL6180xInit();
  sensor.VL6180xDefautSettings(); //Load default settings to get started.

  tcaselect(0); //select channel 0 on the multiplexer
  uint16_t IR = uv.readIR(); //command to read the IR sensor

  tcaselect(1); //select channel 1
  uint16_t IR1 = uv1.readIR(); //command to read the IR sensor


  uint16_t motorASpeed = IR / 4; //divide the reading by a scalar amount to attain a speed between 0-255
  uint16_t motorBSpeed = IR1 / 4;
  
  //set motor speed to min speed if the value is below minSpeed
  if (motorASpeed < minSpeed) {
    motorASpeed = minSpeed;
  }
  if (motorBSpeed < minSpeed) {
    motorBSpeed = minSpeed;
  }

  static uint8_t T = 0; //initialize increment variable
  static uint8_t N = 1; //initialize increment variable
  static uint8_t Q = 0; //initialize increment variable
  static uint16_t PmotorSpeed = 0.5 * (motorASpeed + motorBSpeed); // initializes the variable that stores the motor speed the last time through the loop

  digitalWrite(10, HIGH);
  digitalWrite(14, HIGH);
  digitalWrite(STBY, HIGH);
  digitalWrite(AIN1, HIGH); // configures the left motor A to rotate counter clockwise
  digitalWrite(AIN2, LOW);
  digitalWrite(BIN1, HIGH); // configures the left motor B to rotate clockwise
  digitalWrite(BIN2, LOW);

 //Which Way loop
  while (IR < black && IR1 < black) {
    uint16_t reading = sensor.getDistance(); //read range finder
    if (lastState != isIntersect)
    {
      //call HitWall functions at intersections/new room
      bool Hitwall = HitWall(test);
      bool Leftpaw = LeftPaw(test);
      bool Rightpaw = RightPaw(test);

      if (Hitwall == 0 && Leftpaw == 0 && Rightpaw == 0) // cond 000
      {
        //        Serial.println("cond 000");
        test.turn = 0b00; //no turn
        test.dir = turnInMaze(test);
        test = stepInMaze(test);
        test = roomInMaze(test);
      }
      else if (Leftpaw == 0 && Hitwall == 0 && Rightpaw == 1) //cond 001
      {
        //        Serial.println("cond 001");
        Q++; //increment Q
        if (Q == 1)
        {
          test.turn = 0b00;
        }
        else
        {
          test.turn = 0b10; //Turn Left
          GyroLeft();
        }
        test.dir = turnInMaze(test);
        test = stepInMaze(test);
        test = roomInMaze(test);
        test.turn = 0b00; // reset turn variable to no turn
      }
      else if (Leftpaw == 0 && Hitwall == 1 && Rightpaw == 0) //cond 010
      {
        T++; //increment T
        //        Serial.println("cond 010");
        if (T == 2 || T == 4)
        {
          test.turn = 0b01; //Turn Right
          GyroRight();
        }
        else
        {
          test.turn = 0b10; //Turn Left
          GyroLeft();
        }
        test.dir = turnInMaze(test);
        test = stepInMaze(test);
        test = roomInMaze(test);
        test.turn = 0b00; // reset turn variable to no turn

      }
      else if (Leftpaw == 0 && Hitwall == 1 && Rightpaw == 1) //cond 011
      {
        //        Serial.println("cond 011");
        test.turn = 0b10; //Turn Left
        GyroLeft();
        test.dir = turnInMaze(test);
        test = stepInMaze(test);
        test = roomInMaze(test);
        test.turn = 0b00; // reset turn variable to no turn
      }
      else if (Leftpaw == 1 && Hitwall == 0 && Rightpaw == 0) //cond 100
      {
        N++;
        //        Serial.println("cond 100");
        //        Serial.println(N);
        if (N == 3 || N == 6)
        {
          test.turn = 0b01; //Turn Right
          GyroRight();
        }
        else
        {
          test.turn = 0b00; //no turn
        }
        test.dir = turnInMaze(test);
        test = stepInMaze(test);
        test = roomInMaze(test);
        test.turn = 0b00; // reset turn variable to no turn
      }
      else if (Leftpaw == 1 && Hitwall == 0 && Rightpaw == 1)//cond 101
      {
        //        Serial.println("cond 101");
        test.turn = 0b00; //no turn
        test.dir = turnInMaze(test);
        test = stepInMaze(test);
        test = roomInMaze(test);
      }
      else if (Leftpaw == 1 && Hitwall == 1 && Rightpaw == 0)//cond 110
      {
        //        Serial.println("cond 110");
        test.turn = 0b01; //Turn Right
        GyroRight();
        test.dir = turnInMaze(test);
        test = stepInMaze(test);
        test = roomInMaze(test);
        test.turn = 0b00; // reset turn variable to no turn
      }
      else if (Leftpaw == 1 && Hitwall == 1 && Rightpaw == 1) // cond 111
      {
        //        Serial.println("cond 111");
        test.turn = 0b11; //Turn around
        GyroAround();
        test.dir = turnInMaze(test);
        test = stepInMaze(test);
        test = roomInMaze(test);
        test.turn = 0b00; // reset turn variable to no turn
      }
    }
    else {

      analogWrite(PWMA, PmotorSpeed); // command motors to turn at the previous motor speed from before intersection
      analogWrite(PWMB, PmotorSpeed); // command motors to turn at the previous motor speed from before intersection
      delay(10);
    }
    lastState = isIntersect; // keeps track of isIntersect for the next time through this while loop, used to see if isIntersect changed.

    //update sensor readings
    tcaselect(0);
    IR = uv.readIR();

    tcaselect(1);
    IR1 = uv1.readIR();
  }
  isIntersect = LOW;
  analogWrite(PWMA, motorBSpeed); // command the motors to turn at a given speed
  analogWrite(PWMB, motorASpeed); // command the motors to turn at a given speed
  delay(10);
  PmotorSpeed = 0.5 * (motorASpeed + motorBSpeed);
  lastState = isIntersect; // keeps track of isIntersect for the next time through the main loop, used to see if isIntersect changed.
  analogWrite(PWMA, motorBSpeed); // command the motors to turn at a given speed
  analogWrite(PWMB, motorASpeed); // command the motors to turn at a given speed
  delay(10);
  //  GyroLeft();
  //  delay(2000);
  //  GyroRight();
  //  delay(2000);
  //  GyroAround();
  //        uint16_t reading = sensor.getDistance();
  //  do {
  //    reading = sensor.getDistance();
  //    analogWrite(PWMA, 0);
  //    analogWrite(PWMB, 0);
  //  } while (reading > 120);
  //  if (reading < 120) {
  //    GyroAround();
  //  }
  delay(10);
}


void Init2Winit() {
  //PORT F initialization
  DDRF |= (1 << AIN1) | (1 << AIN2) | (1 << BIN1) | (1 << BIN2); //Output
  PORTF &= ~(1 << AIN1) & ~(1 << AIN2) & ~(1 << BIN1) & ~(1 << BIN2); //Set output to LOW

  //PORT C initialization
  DDRC |= (1 << PWMB); //Output
  PORTC &= ~(1 << PWMB); //Set output to LOW

  //PORT D initialization
  DDRD |= (1 << PWMA) ; //Output
  PORTD &= ~((1 << PWMA) ); //Set output to LOW

  //PORT B initialization

  DDRB |= ((1 << STBY) ); //Output
  PORTB &= ~((1 << STBY) ); //no pullup
}


void GyroLeft() {
  currentAngle = 0; //reset the current angle to start integrating from
  while (currentAngle < 90) { //set to stop turning once integrated angle reaches 90 degrees
    if (then > 0) { //start reading when the amount of time passed is > 0
      zRate = readZ(); //read angular velocity
      zRate /= 14.375; //conversion rate from LSB/ADC value to degrees/second
      if (zRate < 2 && zRate > -2 ) { //gyro has 1 degrees/second drift so unless the gyro is
        //rotating faster than 2 degrees per second, we set the rate to 0 as it is "not moving"
        zRate = 0;
      }
    }
    long delta_time = millis() - then; //takes time the program was started and subtract it from the time now
    digitalWrite(STBY, HIGH);
    digitalWrite(AIN1, LOW);
    digitalWrite(AIN2, HIGH);
    digitalWrite(BIN1, HIGH);
    digitalWrite(BIN2, LOW);
    analogWrite(PWMA, tSpeed);
    analogWrite(PWMB, tSpeed); // motor turning left code
    double delta_angle = zRate * delta_time; //delta theta is just equal to speed * dt
    currentAngle = currentAngle + (delta_angle / 1000); // update the current angle, delta theta is divided
    //by 1000 due to time being in miliseconds
    //Serial.println(currentAngle);
    then = millis(); // reset the time to start recording from end of while loop for accurracy, gives better
    //delta time, the first loop is inaccurate but that's okay
    //because you won't finish integrating by end of first loop
    delay(10);        // 10 second delay because gyro samples at 100Hz and there's no point in recording faster than
    //the gyro can sample
  }
}
void GyroRight() {
  currentAngle = 0;
  while (abs(currentAngle) < 90) {
    if (then > 0) {
      zRate = readZ();
      zRate /= 14.3;
      if (zRate < 2 && zRate > -2 ) {
        zRate = 0;
      }
    }
    long delta_time = millis() - then;
    digitalWrite(STBY, HIGH);
    digitalWrite(AIN1, HIGH);
    digitalWrite(AIN2, LOW);
    digitalWrite(BIN1, LOW);
    digitalWrite(BIN2, HIGH);
    analogWrite(PWMA, tSpeed);
    analogWrite(PWMB, tSpeed);
    double delta_angle = zRate * delta_time;
    currentAngle = currentAngle + (delta_angle / 1000);
    //Serial.println(currentAngle);
    then = millis();
    delay(10);
  }
}
void GyroAround() {
  currentAngle = 0;
  while (currentAngle < 180) {
    if (then > 0) {
      zRate = readZ();
      zRate /= 14.3;
      if (zRate < 2 && zRate > -2 ) {
        zRate = 0;
      }
    }
    long delta_time = millis() - then;
    digitalWrite(STBY, HIGH);
    digitalWrite(AIN1, LOW);
    digitalWrite(AIN2, HIGH);
    digitalWrite(BIN1, HIGH);
    digitalWrite(BIN2, LOW);
    analogWrite(PWMA, tSpeed);
    analogWrite(PWMB, tSpeed);
    double delta_angle = zRate * delta_time;
    currentAngle = currentAngle + (delta_angle / 1000);
    //Serial.println(currentAngle);
    then = millis();
    delay(10);
  }
}

